import java.io.*;
import java.util.*;

import static java.lang.Integer.*;

public class JeroenTranscribedScanner {

    private int n;
    private List<Integer> fact;
    private List<Integer> ifact;

    public static void main(String[] args) throws IOException {
        try (BufferedReader reader = new BufferedReader(new InputStreamReader(System.in))) {
            new JeroenTranscribedScanner().run(new Scanner(reader));
        }
    }

    final int MOD = (int) 1e9 + 7;

    int mul(int a, int b) {
        return (int) (((long) a) * b % MOD);
    }

    int binpow(int a, int b) {
        int res = 1;
        while (b != 0) {
            if (b % 2 == 1) res = mul(res, a);
            b /= 2;
            a = mul(a, a);
        }
        return res;
    }

    int invmod(int v) {
        return binpow(v, MOD - 2);
    }

    int ncr(int a, int b) {
        if (b < 0 || a < b) return 0;
        return mul(mul(fact.get(a), ifact.get(b)), ifact.get(a - b));
    }

    public void run(Scanner sc) throws IOException {
        n = sc.nextInt();
        fact = new ArrayList<>(n + 1);
        ifact = new ArrayList<>(n + 1);
        fact.add(1);
        fact.add(1);
        ifact.add(0);
        ifact.add(0);
        for (int i = 2; i <= n; i++) {
            fact.add(mul(i, fact.get(i - 1)));
            ifact.add(0);
        }
        ifact.set(n, invmod(fact.get(n)));
        for (int i = n - 1; i >= 0; --i) {
            ifact.set(i, mul(i + 1, ifact.get(i + 1)));
        }

        List<Pair> ls = new ArrayList<>(n);
        List<Integer> before = new ArrayList<>(n + 2), after = new ArrayList<>(n + 2);
        for (int i = 0; i < n + 2; i++) {
            before.add(0);
            after.add(0);
        }
        for (int i = 0; i < n; i++) {
            int l = sc.nextInt(), s = sc.nextInt();
            ls.add(new Pair(l, s));
            int mx = max(l, s), mn = min(l, s);
            before.set(mx, before.get(mx) + 1);
            after.set(mn, after.get(mn) + 1);
        }

        for (int i = 1; i < n + 2; i++) {
            before.set(i, before.get(i) + before.get(i - 1));
        }
        for (int i = n; i >= 0; i--) {
            after.set(i, after.get(i) + after.get(i + 1));
        }

        int ans = 0;
        for (Pair pair : ls) {
            int l = pair.l, s = pair.s;
            for (int myscore : new int[]{l, s}) {
                int b = before.get(myscore - 1), a = after.get(myscore);
                if (min(l, s) >= myscore) --a;
                int inbetween = n - 1 - b - a;
                ans += mul(ncr(inbetween, myscore - b - 1), binpow(2, a + b));
                if (ans >= MOD) ans -= MOD;
            }
        }
        System.out.println(ans);
    }

    static class Pair {
        int l, s;

        public Pair(int s, int l) {
            this.s = s;
            this.l = l;
        }

        @Override
        public boolean equals(Object o) {
            if (this == o) return true;
            if (o == null || getClass() != o.getClass()) return false;
            Pair pair = (Pair) o;
            return l == pair.l && s == pair.s;
        }

        @Override
        public int hashCode() {
            return Objects.hash(l, s);
        }

        @Override
        public String toString() {
            return "Pair{l=" + l + ", s=" + s + '}';
        }
    }
}
