import java.io.*;
import java.util.*;

public class MaartenScanner {

    public static void main(String[] args) throws IOException {
        try (BufferedReader reader = new BufferedReader(new InputStreamReader(System.in))) {
            new MaartenScanner(new Scanner(reader)).run();
        }
    }

    private final Scanner sc;
    private int n;
    private double D;
    private List<Point> points;
    private List<Integer> targets;
    private int curr;
    private Stack<Integer> stack;
    private double fuel_used;
    private Stack<Double> fuel_stack;

    public MaartenScanner(Scanner sc) {
        this.sc = sc;
    }

    public void run() throws IOException {
        n = sc.nextInt();
        D = sc.nextInt();
        points = new ArrayList<>(n + 1);
        targets = new ArrayList<>(n + 1);
        points.add(null);
        targets.add(0);
        for (int i = 0; i < n; i++) {
            points.add(new Point(sc.nextInt(), sc.nextInt()));
            targets.add(0);
        }

        curr = 1;
        stack = new Stack<>();
        stack.add(1);
        fuel_used = 0;
        fuel_stack = new Stack<>();
        fuel_stack.add(0.);

        while (true) {
            List<Integer> possible_targets = get_possible_targets(curr);
            if (possible_targets.isEmpty()) {
                List<Integer> back_to_here = null;
                while (curr != 1) {
                    System.out.println(stack.pop());
                    curr = sc.nextInt();
                    fuel_used -= fuel_stack.pop();
                    if (back_to_here != null)
                        back_to_here.add(curr);
                        // Save the first point with unexplored targets on our way back to the start.
                    else if (!get_possible_targets(curr).isEmpty())
                        back_to_here = new ArrayList<>();
                }

                stack.clear();
                stack.add(1);
                fuel_used = 0;
                fuel_stack.clear();
                fuel_stack.add(0.);

                // After moving back to start, move back to the first point where we still have unexplored targets.
                if (back_to_here != null) {
                    back_to_here.remove(back_to_here.size() - 1);
                    Collections.reverse(back_to_here);
                    for (Integer p : back_to_here) {
                        move_to(p);
                    }
                }

                continue;
            }

            if (possible_targets.contains(n)) {
                System.out.println(n);
                return;
            }

            move_to(possible_targets.get(0));
        }
    }

    List<Integer> get_possible_targets(int curr) {
        List<Integer> res = new ArrayList<>(n);
        for (int i = 2; i <= n; i++) {
            if (targets.get(i) == 0 && fuel_used +dist(points.get(curr), points.get(i)) <= (i == n ? D : D / 2))
                res.add(i);
        }
        return res;
    }


    void move_to(int s) {
        System.out.println(s);
        int t = sc.nextInt();
        fuel_stack.add(dist(points.get(curr), points.get(s)));
        fuel_used += fuel_stack.peek();
        targets.set(s, t);
        targets.set(t, s);
        curr = t;
        stack.add(t);
    }

    private double dist(Point a, Point b) {
        return Math.sqrt((a.x - b.x) * (a.x - b.x) + (a.y - b.y) * (a.y - b.y));
    }

    static class Point {
        double x, y;

        public Point(int x, int y) {
            this.x = x;
            this.y = y;
        }

        @Override
        public boolean equals(Object o) {
            if (this == o) return true;
            if (o == null || getClass() != o.getClass()) return false;
            Point point = (Point) o;
            return x == point.x && y == point.y;
        }

        @Override
        public int hashCode() {
            return Objects.hash(x, y);
        }

        @Override
        public String toString() {
            return "Point{x=" + x + ", y=" + y + '}';
        }
    }
}
