private const val MOD =  1000000007

@JvmInline
@Suppress("NOTHING_TO_INLINE")
private value class ModInt(val x: Int) {
    inline operator fun plus(other: ModInt) = ModInt((x + other.x).let { if (it >= MOD) it - MOD else it })
    inline operator fun minus(other: ModInt) = ModInt((x - other.x).let { if (it < 0) it + MOD else it })
    inline operator fun times(other: ModInt) = ModInt((x.toLong() * other.x % MOD).toInt())
    fun power(p_: Int): ModInt {
        var a = this
        var res = ModInt(1)
        var p = p_
        while (p != 0) {
            if ((p and 1) == 1) res *= a
            a *= a
            p = p shr 1
        }
        return res
    }

    inline operator fun div(other: ModInt) = this * other.inv()
    inline fun inv() = power(MOD - 2)

    companion object {
        inline fun from(x: Int) = ModInt((x % MOD + MOD) % MOD)
        inline fun from(x: Long) = ModInt(((x % MOD).toInt() + MOD) % MOD)
    }
}

@JvmInline
private value class ModIntArray(private val storage:IntArray) {
    operator fun get(index: Int) = ModInt(storage[index])
    operator fun set(index: Int, value: ModInt) { storage[index] = value.x }
    val size get() = storage.size
}
private inline fun ModIntArray(n: Int, init: (Int) -> ModInt) = ModIntArray(IntArray(n) { init(it).x })


fun main() {

    val n = readInt()
    val COMB_MAX = n + 1
    val fs = ModIntArray(COMB_MAX) { ModInt(1) }.apply {
        for (i in 1 until size) {
            set(i, get(i - 1) * ModInt.from(i))
        }
    }
    val ifs = ModIntArray(COMB_MAX) { fs[it].inv() }
    fun cnk(n: Int, k: Int) = fs[n] * ifs[k] * ifs[n - k]

    val data = List(n) { readInts() }

    val a = buildList {
        repeat(n) {
            val (x, y) = data[it]
            add(x to it)
            add(y to it)
        }
    }.groupBy { it.first }.entries.sortedBy { it.key }.map { it.value }
    val cnt = IntArray(n)
    var os = 0
    var ts = 0
    var ans = ModInt(0)
    for (lst in a) {
        for ((v, id) in lst) {
            require(cnt[id] < 2)
            val need = (v - 1) - ts
            val cos = os - if (cnt[id] == 1) 1 else 0
            if (need in 0 .. cos) {
                ans += cnk(cos, need) * ModInt(2).power(n - cos - 1)
            }
        }
        for ((_, id) in lst) {
            cnt[id]++
            if (cnt[id] == 1) os++
            if (cnt[id] == 2) { os--; ts++ }
        }
    }
    print(ans.x)

}

private fun readInt() = readln().toInt()
private fun readLongs() = readStrings().map { it.toLong() }
private fun readStrings() = readln().split(" ")
private fun readInts() = readStrings().map { it.toInt() }