#!/usr/bin/env python3


class Trie(dict):
    max_depth = -1

    def __repr__(self):  # Debugging purposes
        return super().__repr__() + f"[max_depth={self.max_depth}]"


trie = Trie({})
for word in set(input() + "." for _ in range(int(input()))):
    curr = trie
    for i, c in enumerate(word):
        node = curr.get(c)
        if isinstance(node, Trie):
            curr = node
        elif isinstance(node, list):
            last = node.pop()
            curr[c] = Trie({last: node})
            curr = curr[c]
        elif node is None:
            # We store the remainder of the word in reverse, so we can do an O(1) .pop() later
            curr[c] = list(reversed(word[i + 1:]))
            break
        else:
            raise Exception(f"Invalid trie node: {node}")

stack = [trie]
while stack:
    curr = stack.pop()
    if all(isinstance(node, list) or node.max_depth != -1 for node in curr.values()):
        curr.max_depth = 1 + max(len(node) if isinstance(node, list) else node.max_depth for node in curr.values())
        continue

    stack.append(curr)
    for node in curr.values():
        if isinstance(node, Trie):
            stack.append(node)

ans, stack = 0, [trie]
while stack:
    curr = stack.pop()
    for node in curr.values():
        if isinstance(node, Trie):
            stack.append(node)

    if len(curr) == 1:
        continue
    ans = max(ans, sum(sorted(len(node) if isinstance(node, list) else node.max_depth for node in curr.values())[-2:]))

print(ans)
