#include <vector>
#include <utility>
#include <queue>
#include <iostream>
#include <cmath>

const double INF = 1e6 + 1;

struct Graph {
    std::vector<std::vector<std::pair<int, double>>> edges;
    std::vector<double> dist;
    std::vector<int> father;

    explicit Graph(int n): edges(1 + n), dist(1 + n), father(1 + n) {}

    void dijkstra() {
        std::priority_queue<std::pair<double, int>> pq;

        for (int i = 0; i < dist.size(); i++)
            dist[i] = INF;

        pq.push({0.0f, 1});
        dist[1] = 0.0f;

        while (!pq.empty()) {
            int node = pq.top().second;
            double d = -pq.top().first;

            pq.pop();

            if (d != dist[node])
                continue;

            for (auto it: edges[node]) {
                if (d + it.second < dist[it.first]) {
                    dist[it.first] = d + it.second;
                    father[it.first] = node;
                    pq.push({-dist[it.first], it.first});
                }
            }
        }
    }

    std::vector<int> get_chain(int n) {
        std::vector<int> chain;
        while (n != 1) {
            chain.push_back(n);
            n = father[n];
        }
        chain.push_back(1);
        return chain;
    }
};

double dist(std::pair<int, int> a, std::pair<int, int> b) {
    return std::sqrt((long long)(a.first - b.first) * (a.first - b.first) +
                     (long long)(a.second - b.second) * (a.second - b.second));
}

int travel(int to, int n) {
    std::cout << to << std::endl;
    
    if (to == n) exit(0);

    int val;
    std::cin >> val;
    return val;
}

int main() {
    int n, d;

    std::cin >> n >> d;

    std::vector<std::pair<int, int>> points(n);

    Graph G(n);

    for (int i = 0; i < n; i++) {
        std::cin >> points[i].first >> points[i].second;
        for (int j = 0; j < i; j++) {
            double dij = dist(points[i], points[j]);
            G.edges[i + 1].push_back({j + 1, dij});
            G.edges[j + 1].push_back({i + 1, dij});
        }
    }

    G.dijkstra();

    std::vector<bool> discovered(1 + n);
    discovered[1] = true;

    while (true) {
        // choose some wormhole to discover
        int chosen = 1;
        for (int i = 2; i <= n; i++)
            if (!discovered[i] && G.dist[i] * 2 <= d)
                chosen = i;
        
        std::vector<int> chain = G.get_chain(chosen);
        
        int node = 1;
        for (int t = chain.size() - 1; t >= 0; t--) {
            if (node != chain[t])
                node = travel(chain[t], n);
        }

        // there is a new edge between chosen and node
        G.edges[node].push_back({chosen, 0});
        G.edges[chosen].push_back({node, 0});
   
        discovered[chosen] = discovered[node] = true;

        // Return back to the end of the chain and go back to node 1
        node = travel(node, n);
        for (auto it: chain)
            if (it != node)
                node = travel(it, n);

        G.dijkstra();
    }

    return 0;
}

