/*
Interresting graph problem where using the given guarantees on the input has some interresting deductions.
Basicly do a DFS  on nodes in range with fuel availble and greedy look for reaching the relic after every warp, but refuel if the current location can be faster reached directly from the start (and recheck the greedy move to the relic).
Implementing the backtrack algorithm can be tricky. In my implementation, I always refuel first and then go back to the refuel point.
It might be possible a test case exists where this approach leads to many refeuls, but I can't think one.
 */

import kotlin.math.pow
import kotlin.math.sqrt

fun main() {
    val input = System.`in`.bufferedReader()
    val output = System.out.bufferedWriter()
    val (n, d) = input.readLine().split(" ").map { it.toInt() }
    val stations = mutableListOf<Coordinate>()
    repeat(n) {
        val (x, y) = input.readLine().split(" ").map { it.toDouble() }
        stations.add(x to y)
    }
    val galaxyMap = stations.mapIndexed { i, c -> i + 1 to c }.toMap()
    val visited = mutableSetOf(1)
    var currentFuel = d.toDouble()
    val refuel = currentFuel / 2

    fun writeOutput(location: Int): Int {
        output.write("$location")
        output.newLine()
        output.flush()
        if (location == n) return n
        val ret = input.readLine().toInt()
        return ret
    }

    // calculate a distance map
    val distance = Array(n + 1) { from ->
        DoubleArray(n + 1) {
            if (from == 0) return@DoubleArray Double.MAX_VALUE
            when (it) {
                0 -> Double.MAX_VALUE
                from -> 0.0
                else -> distance(galaxyMap[from]!!, galaxyMap[it]!!)
            }
        }
    }
    if (distance[1][n] < currentFuel) {
        writeOutput(n)
        return
    }
    val inRange =
        (1..n).associateWith { from -> distance[from].withIndex().filter { it.value <= refuel } }
            .toMutableMap()


    var location = 1
    val path = mutableListOf<Pair<Int, Double>>()

    while (location != n) {
        if (location == 1) {
            currentFuel = d.toDouble()
            path.clear()
        }
        val option = inRange[location]!!
            .filter { it.index !in visited }
            .minByOrNull { it.value }
        if (option == null || currentFuel - option.value < refuel) {
            // backtrack
            var backTrackFound = false
            val returnPath = mutableListOf<Int>()
            path.reversed().windowed(2, step = 2).forEach { (to, from) ->
                val (step, fuel) = to
                writeOutput(step)
                if (!backTrackFound && step != 1 && inRange[step]!!.any { it.index !in visited }) {
                    backTrackFound = true
                    location = step
                    currentFuel = fuel
                }
                if (backTrackFound) {
                    returnPath.add(from.first)
                } else {
                    // since we are not coming back here, remove
                    path.removeLast()
                    path.removeLast()
                }
            }
            writeOutput(1)
            if (!backTrackFound) {
                location = 1
            } else {
                returnPath.reversed().forEach {
                    writeOutput(it)
                }
            }
            continue
        }
        val (next, dist) = option
        currentFuel -= dist
        visited.add(next)
        path.add(next to currentFuel)
        val result = writeOutput(next)
        if (next == n) return
        location = result
        // update inRange
        inRange[result] = inRange[result]!!.filter { it.index !in visited }.filter { it.value <= currentFuel }
        path.add(result to currentFuel)
        visited.add(result)
        // can we reach the relic with our current fuel?
        if (distance[result][n] < currentFuel) {
            writeOutput(n)
            return
        }
        // check if refeuling and flying back nets us with more fuel
        val potential = d.toDouble() - distance[result][1]
        if (potential > currentFuel) {
            // System.err.println("Refeuling: $currentFuel to ${potential}(${distance[result][1]})")
            currentFuel = potential
            writeOutput(1)
            writeOutput(result) // warps us back to next
            writeOutput(next)
        }
        // can we reach the relic with our current fuel?
        if (distance[result][n] < currentFuel) {
            writeOutput(n)
            return
        }
    }
}

fun distance(a: Coordinate, b: Coordinate) = sqrt((b.first - a.first).pow(2) + (b.second - a.second).pow(2))
typealias Coordinate = Pair<Double, Double>