#include <iostream>
#include <memory>

using namespace std;

bool imp(bool x, bool y) { return ((not x) or y); }

struct Node {
    int l = 0, r = 0;
    bool res = false; // (((pl -> pl+1) -> pl+2) -> ... )...) -> pr
    bool case_true = false; // (1 -> ((pl -> pl+1) -> pl+2) -> ... )...) -> pr
    bool case_false = false; // (0 -> ((pl -> pl+1) -> pl+2) -> ... )...) -> pr
    shared_ptr<Node> left_child = nullptr;
    shared_ptr<Node> right_child = nullptr;
    Node (int l, int r, bool res, bool case_true, bool case_false) 
        : l{l}, r{r}, res{res}, case_true{case_true}, case_false{case_false} {}

    tuple<bool,bool,bool> query(int lq, int rq) {
        if (l == lq and r == rq) {
            return {res, case_true, case_false};
        } else {
            int m = l + (r - l)/2;
            if (rq <= m) return left_child->query(lq, rq);
            if (lq >= m+1) return right_child->query(lq, rq);
            else {
                auto [resL, case_trueL, case_falseL] = left_child->query(lq, m);
                auto [resR, case_trueR, case_falseR] = right_child->query(m+1, rq);
                return {
                    (resL ? case_trueR : case_falseR),
                    (case_trueL ? case_trueR : case_falseR),
                    (case_falseL ? case_trueR : case_falseR)
                };
            } 
        }
    }

    void print(int d = 1) {
        cout << "[" << l << "," << r << "]" << endl; 
        cout << res << case_true << case_false << endl;
        if (left_child) {
            for (int j = 0; j < d; j++)
                cout << "-";
            left_child->print(d+1);
        }
        if (right_child) {
            for (int j = 0; j < d; j++)
                cout << "-";
            right_child->print(d+1);
        }
    }
};

shared_ptr<Node> make_tree(int l, int r, bool* v) {
    if (l == r) {
        return make_shared<Node>(l, r, v[l], imp(true, v[l]), imp(false, v[l]));
    } else {
        int rc = l + (r - l) / 2;
        auto left_child = make_tree(l, rc, v);
        auto right_child = make_tree(rc+1, r, v);

        bool res = (left_child->res) ? (right_child->case_true) : (right_child->case_false); 
        bool case_true = left_child->case_true ? right_child->case_true : right_child->case_false;
        bool case_false = left_child->case_false ? right_child->case_true : right_child->case_false;

        auto node =  make_shared<Node>(l, r, res, case_true, case_false);
        node->left_child = left_child;
        node->right_child = right_child;
        return node;
    }
}

int main() {
    int n, q;
    cin >> n;
    bool v[n];
    for (int i = 0; i < n; i++)
        cin >> v[i];
    auto root = make_tree(0, n-1, v);
    cin >> q;
    for (int i = 0; i < q; i++) {
        int lq, rq;
        cin >> lq >> rq;
        cout << get<0>(root->query(lq-1, rq-1)) << endl;
    }
    return 0;
}
