#include <iostream>
#include <vector>
#include <cmath>
#include <stack>
#include <unordered_set>
#include <unordered_map>
#include <queue>
#include <algorithm>

struct Point {
	int x, y;

	[[nodiscard]] double distance(const Point other) const {
		return std::sqrt(static_cast<double>(x - other.x) * static_cast<double>(x - other.x) + static_cast<double>(y - other.y) * static_cast<double>(y - other.y));
	}

	// Who doesn't love the spaceship operator.
	// Thematic isn't it...
	auto operator<=>(const Point&) const = default;
	/*bool operator==(A const & other) const {
		return this->_equals(other);
	}

	bool operator!=(A const & other) const {
		return !(*this == other);
	}*/
};

// Derived from: https://stackoverflow.com/a/17017281/6431494
template<>
struct std::hash<Point> {
	std::size_t operator()(const Point& k) const {
		std::size_t ans = 17;
		ans = ans * 31 + std::hash<int>()(k.x);
		ans = ans * 31 + std::hash<int>()(k.y);
		return ans;
	}
};

template<>
struct std::hash<std::pair<Point, Point>> {
	std::size_t operator()(const std::pair<Point, Point>& k) const {
		std::size_t ans = 17;
		ans = ans * 31 + std::hash<Point>()(k.first);
		ans = ans * 31 + std::hash<Point>()(k.second);
		return ans;
	}
};

auto points = std::vector<Point>();
auto point2index = std::unordered_map<Point, unsigned int>();
std::unordered_map<std::pair<Point, Point>, double> distances;

std::vector<Point> recover_path(const Point end, std::unordered_map<Point, Point> parent) {
	auto path = std::vector<Point>();
	auto curr = end;
	for(;;) {
		path.push_back(curr);
		if(parent[curr] == curr) {
			break;
		}
		curr = parent[curr];
	}
	path.pop_back();
	std::ranges::reverse(path);
	return path;
}

std::pair<std::vector<Point>, double> find_path(Point start, double fuel, std::unordered_set<Point> ends) {
	auto dist = std::unordered_map<Point, double>();
	dist[start] = 0;

	auto q = std::priority_queue<
		std::pair<Point, double>,
		std::vector<std::pair<Point, double>>,
		decltype([](auto& a, auto& b) { return a.second > b.second; })
	>();
	q.emplace(start, 0);

	auto parents = std::unordered_map<Point, Point>();
	parents[start] = start;

	while(!q.empty()) {
		auto [point, old_dist] = q.top();
		q.pop();

		if(ends.contains(point)) {
			return { recover_path(point, parents), dist[point] };
		}

		// If the distance is actually old
		// Then we've already been through this point
		// Through a more optimal path
		// So we skip it
		if(dist[point] != old_dist) {
			continue;
		}

		if(dist[point] > fuel) {
			return {{}, dist[point]};
		}

		for(int i = 1; i < points.size(); ++i) {
			auto nb = points[i];
			if(nb == point) {
				continue;
			}
			if(!dist.contains(nb) || dist[point] + distances[{point, nb}] < dist[nb]) {
				dist[nb] = dist[point] + distances[{point, nb}];
				parents[nb] = point;
				q.push({nb, dist[nb]});
			}
		}
	}

	std::cout << "This shouldn't happen\n";
	return {};
}

Point execute_path(Point start, const std::vector<Point>& path) {
	int otherside;
	Point last = start;
	for(int i = 0; i < path.size(); ++i) {
		if(distances[{last, path[i]}] == 0) {
			std::cout << point2index[last] << std::endl;
			last = path[i];
			int _;
			std::cin >> _;
			continue;
		}
		std::cout << point2index[path[i]] << std::endl;
		if(point2index[path[i]] == point2index.size()) {
			// We've reached the relic, so we shouldn't expect any more inputs
			return path[i];
		}
		std::cin >> otherside;
		std::cout << otherside << std::endl;
		int _;
		std::cin >> _;
		last = path[i];
	}
	return points[otherside];
}

int main() {
	int n, d;
	std::cin >> n >> d;

	points.push_back(Point());

	for(int i = 1; i <= n; ++i) {
		int x, y;
		std::cin >> x >> y;
		points.push_back(Point(x, y));
		point2index[Point(x, y)] = i;
	}

	for(auto i : points) {
		for(auto j : points) {
			distances[{i, j}] = i.distance(j);
		}
	}

	auto unvisited = std::unordered_set<Point>();

	for(int i = 2; i <= n; ++i) {
		unvisited.insert(points[i]);
	}

	auto relic = points[points.size() - 1];
	auto home_base = points[1];
	auto spaceship = home_base;
	double fuel = d;

	// Caching the path to home base
	auto _path_home = std::vector<Point>();
	auto _dist_home = 0;

	while(!unvisited.empty()) {

		auto [path_to_relic, distance_to_relic] = find_path(spaceship, fuel, {relic});
		if(distance_to_relic < fuel) {
			execute_path(spaceship, path_to_relic);
			// That's it. We've reached the relic. Hooray.
			return 0;
		}


		auto path_to_home_base = _path_home;
		auto distance_to_home_base = _dist_home;
		// Too slow if we don't cache
		//auto [path_to_home_base, distance_to_home_base] = find_path(spaceship, fuel, {home_base});
		auto [path_to_closest_unvisited, distance_to_closest_unvisited] = find_path(spaceship, fuel, unvisited);

		// Can we afford to go to the wormhole and go back?
		if(distance_to_home_base + 2 * distance_to_closest_unvisited < fuel) {
			// Go to closes unvisited wormhole
			auto otherside = execute_path(spaceship, path_to_closest_unvisited);
			auto thisside = path_to_closest_unvisited[path_to_closest_unvisited.size() - 1];
			distances[{thisside, otherside}] = 0;
			distances[{otherside, thisside}] = 0;
			unvisited.erase(thisside);
			unvisited.erase(otherside);
			auto old_spaceship = spaceship;
			spaceship = thisside;
			fuel -= distance_to_closest_unvisited;

			path_to_closest_unvisited.pop_back();
			std::ranges::reverse(path_to_closest_unvisited);
			path_to_closest_unvisited.push_back(old_spaceship);
			for(auto p : _path_home) {
				path_to_closest_unvisited.push_back(p);
			}
			_path_home = path_to_closest_unvisited;
			_dist_home += distance_to_closest_unvisited;
		} else {
			// Go back to home base
			execute_path(spaceship, path_to_home_base);
			spaceship = home_base;
			fuel = d;
			_dist_home = 0;
			_path_home = std::vector<Point>();
		}

	}
}
