// @EXPECTED_RESULTS@: ACCEPTED, TIME_LIMIT_EXCEEDED
// For some reason, on DOMjudge, it runs in 4 seconds on 18-common.in?
/*
Seems like a traditional trie problem to me. I use a priority queue to quickly the longest word size in a node.
I had several issues while implementing. The first was a stack overflow where I added through a recursive call in my trie.
Then I had an OOM for using Strings, which I transferred to a linked list to optimize.

I'm sort of surprised that the search function does not end up in a stack overflow situation, but I tested with the
worse possible case with the generate function (2 equal strings of max length), and it runs fine.

In the end this one was very hard to implement, many corner cases that can go wrong. It's a tricky problem, I expect most
teams to try it, but most will lose a lot of time on it.
 */
import java.util.*

fun main() {
    val trie = TrieNode('^')
    System.`in`.bufferedReader().lineSequence().drop(1).forEach {
            val word = LinkedList<Char>()
            word.add('^')
            word.addAll(it.toList())
            word.add('$')
            add(trie, word)
        }
    println(trie.find())
}

fun add(base: TrieNode, raw: LinkedList<Char>) {
    var trie = base
    var word = raw
    word.removeFirst()
    while (true) {

        trie.words.add(word.size)
        if (trie.leaf) {
            if (trie.value == '$') return
            if (trie.unprocessed.isEmpty()) {
                trie.unprocessed = word
                return
            }
            trie.leaf = false
            val letter = word.first
            trie.children[letter] = TrieNode(word)

            if (trie.unprocessed.first() != letter) {
                trie.children[trie.unprocessed.first()] = TrieNode(trie.unprocessed)
                return
            }
            word = trie.unprocessed
            word.removeFirst()
            trie = trie.children[letter]!!

            continue
        }
        if (trie.children.containsKey(word.first())) {
            trie = trie.children[word.first()]!!
            word.removeFirst()
            continue
        } else {
            trie.children[word.first()] = TrieNode(word)
            return
        }
    }
}

data class TrieNode(
    val value: Char,
    var leaf: Boolean = true,
    val children: MutableMap<Char, TrieNode> = mutableMapOf(),
    var unprocessed: LinkedList<Char> = LinkedList(),
    val words: PriorityQueue<Int> = PriorityQueue { o1, o2 -> o2.compareTo(o1) }
) {
    constructor(word: LinkedList<Char>) : this(word.first()) {
        word.removeFirst()
        unprocessed = word
        words.add(unprocessed.size)
    }


    fun find(): Int {
        if (leaf) {
            return unprocessed.size
        }
        if (children.size == 1) {
            return children[children.keys.first()]!!.find()
        }
        val best = children.values.map { it.words.peek() }.sortedDescending().let { (l1, l2) -> l1 + l2 }
        return children.values.maxOf { it.find() }.coerceAtLeast(best)
    }
}
// helper function to test worse case stack size.
fun generate() {
    println(2)
    repeat(2000000) {
        print('a')
    }
    println()
    repeat(2000000) {
        print('a')
    }
}
