#!/usr/bin/env python3
# @EXPECTED_RESULTS@: RUN_TIME_ERROR, TIME_LIMIT_EXCEEDED
from sys import setrecursionlimit

# Stack size grows way too big, causing memory errors 🙃 May also TLE if runner is slow.
setrecursionlimit(int(1e6))


class Trie(dict):
    max_depth = 0

    def __repr__(self):  # Debugging purposes
        return super().__repr__() + f"[max_depth={self.max_depth}]"


trie = Trie({})
for word in set(input() + "." for _ in range(int(input()))):
    curr = trie
    for i, c in enumerate(word):
        node = curr.get(c)
        if isinstance(node, Trie):
            curr = node
        elif isinstance(node, list):
            last = node.pop()
            curr[c] = Trie({last: node})
            curr = curr[c]
        elif node is None:
            # We store the remainder of the word in reverse, so we can do an O(1) .pop() later
            curr[c] = list(reversed(word[i + 1:]))
            break
        else:
            raise Exception(f"Invalid trie node: {node}")


def calc_depth(trie):
    if isinstance(trie, list):
        return len(trie)
    trie.max_depth = 1 + max(calc_depth(node) for node in trie.values())
    return trie.max_depth


def get_answer(trie):
    if isinstance(trie, list):
        return 0
    if len(trie) == 1:
        return get_answer(list(trie.values()).pop())
    best = sum(sorted(len(node) if isinstance(node, list) else node.max_depth for node in trie.values())[-2:])
    return max(best, *(get_answer(node) for node in trie.values()))


while isinstance(trie, dict) and len(trie) == 1:
    trie = list(trie.values()).pop()
calc_depth(trie)
print(get_answer(trie))
