#include <iostream>
#include <vector>
#include <numeric>
#include <bitset>
#include <cmath>
#include <algorithm>
using namespace std;
#define rep(i,a,b) for(int i=(a); i<(b); ++i)
#define all(x) x.begin(),x.end()
typedef long long ll;
typedef vector<int> vi;
typedef vector<vi> vvi;

inline int modmul(int a, int b, int M){
    return (long long ) a*b%M;
}
inline ll modmul(ll a, ll b, ll M) {
    ll ret = a * b - M * ll(1.L / M * a * b);
    return ret + M * (ret < 0) - M * (ret >= (ll)M);
}
template<typename T>
inline T modpow(T b, T e, T mod) {
    T ans = 1;
    for (; e; b = modmul(b, b, mod), e /= 2)
        if (e & 1) ans = modmul(ans, b, mod);
    return ans;
}

const int prime_factor_cutoff = 650000000;
const int prime_check_cutoff_quart = 500000;
const int prime_check_cutoff_half = prime_check_cutoff_quart*2;
const int prime_check_cutoff = prime_check_cutoff_half*2;

vector<int> prime_table;
bitset<prime_check_cutoff_half> is_composite_table_odd = 1;

void build_prime_table(){
    prime_table.push_back(2);
    for (int i=1; i<prime_check_cutoff_half; ++i){
        if (!is_composite_table_odd[i]){
            int off = i*2 + 1 ;
            prime_table.push_back(off);
            for (int j = i+off; j<prime_check_cutoff_half; j+=off) {
                is_composite_table_odd[j] = 1;
            }
        }
    }
}

template<typename T>
bool isPrime(T p){
    if (p < 2 || p % 6 % 4 != 1) return (p | 1) == 3;
    if (p < prime_check_cutoff) return !is_composite_table_odd[p/2];
    T A[] = {2, 325, 9375, 28178, 450775, 9780504, 1795265022};
    T s = __builtin_ctzll(p-1), d = p >> s;
    for (T a : A) {   // ^ count trailing zeroes
        T q = modpow(a%p, d, p), i = s;
        while (q != 1 && q != p - 1 && a % p && i--) q = modmul(q, q, p);
        if (q != p-1 && i != s) return 0;
    }
    return 1;
}

template<typename T>
T pollard(T n) {
    auto f = [n](T x) { return modmul(x, x, n) + 1; };
    T x = 0, y = 0, t = 30, prd = 2, i = 1, q;
    while (t++ % 40 || gcd(prd, n) == 1) {
        if (x == y) x = ++i, y = f(x);
        if ((q = modmul(prd, max(x,y) - min(x,y), n))) prd = q;
        x = f(x), y = f(f(y));
    }
    return gcd(prd, n);
}

template<typename T>
void factor_sqrt_uth(int n, vector<T>& out){
    int mx = sqrt(n*1.00001f);
    for (int i=0; prime_table[i]<=mx; ++i){
        if (n%prime_table[i]==0){
            do{
                out.push_back(prime_table[i]);
                n /= prime_table[i];
            } while (n%prime_table[i]==0);
            mx = sqrt(n*1.00001f);
        }
    }
    if (n > 1) out.push_back(n);
}

template<typename T>
void factor_uth(T n, vector<T>& out) {
    if (n == 1) return;
    if (isPrime<T>(n)) {
        out.push_back(n);
        return;
    }
    if (n < prime_factor_cutoff){
        factor_sqrt_uth(n, out);
        return;
    }
    T x = pollard<T>(n);
    factor_uth(x, out);
    factor_uth(n / x, out);
}

template<typename T>
vector<T> factor(T n){
    vector<T> ans;
    factor_uth(n, ans);
    if (n >= prime_factor_cutoff) sort(ans.begin(),ans.end()); // optional
    return ans;
}

template<typename T>
vector<pair<T,int>> factor_count(T n){
    vector<T> tmp;
    factor_uth(n, tmp);
    if (n >= prime_factor_cutoff) sort(tmp.begin(),tmp.end());
    vector<pair<T,int>> ans;
    int i=0;
    while (i<tmp.size()){
        int j=i;
        while(j<tmp.size() and tmp[i]==tmp[j]) ++j;
        ans.push_back({tmp[i],j-i});
        i=j;
    }
    return ans;
}

template<typename T>
vector<T> divisors(T n){
    vector<pair<T,int>> tmp = factor_count(n);
    vector<T> ans = {1};
    for (auto [p,e] : tmp){
        auto ans_size = ans.size();
        for (int i=0; i<ans_size; ++i){
            int j=0;
            T cur = ans[i];
            do{
                cur *= p;
                j++;
                ans.push_back(cur);
            } while(j<e);
        }
    }
    sort(ans.begin(),ans.end()); //optional
    return ans;
}

template<typename T>
T eulerphi(T n){
    vector<pair<T,int>> tmp = factor_count(n);
    for (auto [p,e] : tmp) n = (p-1)*(n/p);
    return n;
}

int main(){
   // :::: INCLUDE IN MAIN ::::
    build_prime_table();

    int k; cin >> k;
    while(k--){
        ll n,x; cin >> n >> x;
        if (gcd(n,x) != 1){
            cout << "impossible\n";
            continue;
        }if (x==1){
            cout << "0\n";
            continue;
        }
        ll ord = eulerphi(n);
        auto res = factor_count(ord);
        for (auto [p,e] : res){
            while (ord%p == 0 and modpow(x,ord/p,n) == 1 and e--) ord/=p;
        }
        if (__builtin_popcountll(ord)==1){
            cout << log2(ord) << '\n';
        }else{
            cout << "impossible\n";
        }
    }
}
