#!/usr/bin/env python3

class Trie(dict):
    max_length = -1

    def __repr__(self):  # Debugging purposes
        return super().__repr__() + f"[max_length={self.max_length}]"


trie = Trie({})
for word in set(input() + "." for _ in range(int(input()))):
    curr = trie
    for i, c in enumerate(word):
        node = curr.get(c)
        if isinstance(node, dict):
            curr = node
        elif isinstance(node, list):
            last = node.pop()
            curr[c] = Trie({last: node})
            curr = curr[c]
        elif node is None:
            # We store the remainder of the word in reverse, so we can do an O(1) .pop() later
            curr[c] = list(reversed(word[i + 1:]))
            break
        else:
            raise Exception(f"Invalid trie node: {node}")

stack = [trie]
while stack:
    curr = stack.pop()
    if all(isinstance(node, list) or node.max_length != -1 for node in curr.values()):
        curr.max_length = 1 + max(len(node) if isinstance(node, list) else node.max_length for node in curr.values())
        continue

    stack.append(curr)
    for node in curr.values():
        if isinstance(node, Trie):
            stack.append(node)

while isinstance(trie, dict) and len(trie) == 1:
    trie = list(trie.values()).pop()

print(0 if isinstance(trie, list) else
      sum(sorted(len(node) if isinstance(node, list) else node.max_length for node in trie.values())[-2:]))
