import kotlin.math.pow
import kotlin.math.sqrt

fun main() {
    val input = System.`in`.bufferedReader()
    val output = System.out.bufferedWriter()
    val (n, d) = input.readLine().split(" ").map { it.toInt() }
    System.err.println("We have $n nodes and $d fuel")
    val stations = mutableListOf<Coordinate>()
    repeat(n) {
        val (x, y) = input.readLine().split(" ").map { it.toDouble() }
        stations.add(x to y)
    }
    val galaxyMap = stations.mapIndexed { i, c -> i + 1 to c }.toMap()
    val visited = mutableSetOf(1)
    var currentFuel = d.toDouble()
    val refuel = currentFuel / 2

    fun writeOutput(location: Int) {
        output.write("$location")
        output.newLine()
        output.flush()
    }

    // calculate a distance map
    val distance = Array(n + 1) { from ->
        DoubleArray(n + 1) {
            if (from == 0) return@DoubleArray Double.MAX_VALUE
            when (it) {
                0 -> Double.MAX_VALUE
                from -> 0.0
                else -> distance(galaxyMap[from]!!, galaxyMap[it]!!)
            }
        }
    }
    if(distance[1][n] < currentFuel) {
        writeOutput(n)
        return
    }
    val inRange =
        (1..n).associateWith { from -> distance[from].withIndex().filter { it.value <= refuel } }




    var location = 1
    val path = mutableListOf<Pair<Int, Double>>()

    while (location != n) {
        if (location == 1) {
            currentFuel = d.toDouble()
            path.clear()
            path.add(1 to currentFuel)
        }
        val option = inRange[location]!!
            .filter { it.index !in visited }
            .minByOrNull { it.value }
        if(option == null || currentFuel - option.value < refuel) {
            System.err.println("Starting backtrack at $location with $currentFuel")
            // backtrack
            System.err.println("Backtracking path: $path")
            var backTrackFound = false
            val returnPath = mutableListOf<Int>()
            path.reversed().forEach { (step, fuel) ->
                writeOutput(step)
                if(!backTrackFound && step != 1 && inRange[step]!!.any{it.index !in visited }) {
                    System.err.println("Backtrack found at $step with $fuel with options ${inRange[step]!!.filter{it.index !in visited }}")
                    backTrackFound = true
                    location = step
                    currentFuel = fuel
                } else if(backTrackFound) {
                    returnPath.add(step)
                }
            }
            System.err.println("Ending backtrack at $location with $currentFuel ($backTrackFound, $returnPath)")
            if(!backTrackFound) {
                location = 1
            } else {
                System.err.println("Starting refuel with $returnPath")
                returnPath.reversed().drop(1).forEach {
                    writeOutput(it)
                    readln()
                }
            }
            System.err.println("Result after backtracking: $location $currentFuel")
            continue
        }
        val (next, dist) = option
        System.err.println("Closest is $next with distance $dist")
        currentFuel -= dist
        visited.add(next)
        path.add(next to currentFuel)
        writeOutput(next)
        if (next == n) return
        val result = input.readLine().toInt()
        // can we reach the relic with our current fuel?
        if(distance[result][n] < currentFuel) {
            writeOutput(n)
            return
        }
        // check if refeuling and flying back nets us with more fuel
        if(d.toDouble() - distance[next][1] > currentFuel) {
            System.err.println("Refeuling")
            currentFuel = d.toDouble() - distance[next][1]
            writeOutput(1)
            readln()
            writeOutput(result) // warps us back to next
            readln()
            writeOutput(next)
        }
        // can we reach the relic with our current fuel?
        if(distance[result][n] < currentFuel) {
            writeOutput(n)
            return
        }

        // update inRange
        inRange[result]!!.filter { it.index !in visited }.filter { it.value <= currentFuel }


        location = result
        path.add(result to currentFuel)
        visited.add(result)


    }

}

fun distance(a: Coordinate, b: Coordinate) = sqrt((b.first - a.first).pow(2) + (b.second - a.second).pow(2))
typealias Coordinate = Pair<Double, Double>